@@ -113,7 +113,7 @@ void
 show_revocation_reason (ctrl_t ctrl, PKT_public_key *pk, int mode)
 {
     /* Hmmm, this is not so easy because we have to duplicate the code
-     * used in the trustdb to calculate the keyflags.  We need to find
+     * used in the trustbd to calculate the keyflags.  We need to find
      * a clean way to check revocation certificates on keys and
      * signatures.  And there should be no duplicate code.  Because we
      * enter this function only when the trustdb told us that we have
@@ -548,7 +548,7 @@ check_signatures_trust (ctrl_t ctrl, PKT_signature *sig)
   unsigned int trustlevel = TRUST_UNKNOWN;
   int rc=0;
 
-  rc = get_pubkey_for_sig (ctrl, pk, sig, NULL);
+  rc = get_pubkey (ctrl, pk, sig->keyid );
   if (rc)
     { /* this should not happen */
       log_error("Ooops; the key vanished  - can't check the trust\n");
@@ -834,8 +834,7 @@ find_and_check_key (ctrl_t ctrl, const char *name, unsigned int use,
   if (from_file)
     rc = get_pubkey_fromfile (ctrl, pk, name);
   else
-    rc = get_best_pubkey_byname (ctrl, GET_PUBKEY_NORMAL,
-                                 NULL, pk, name, &keyblock, 0);
+    rc = get_best_pubkey_byname (ctrl, NULL, pk, name, &keyblock, 0, 0);
   if (rc)
     {
       int code;
@@ -976,8 +975,8 @@ build_pk_list (ctrl_t ctrl, strlist_t rcpts, PK_LIST *ret_pk_list)
           r->pk = xmalloc_clear (sizeof *r->pk);
           r->pk->req_usage = PUBKEY_USAGE_ENC;
 
-          rc = get_pubkey_byname (ctrl, GET_PUBKEY_NO_AKL,
-                                  NULL, r->pk, default_key, NULL, NULL, 0);
+          rc = get_pubkey_byname (ctrl, NULL, r->pk, default_key,
+                                   NULL, NULL, 0, 1);
           if (rc)
             {
               xfree (r->pk);
@@ -1042,8 +1041,8 @@ build_pk_list (ctrl_t ctrl, strlist_t rcpts, PK_LIST *ret_pk_list)
           /* We explicitly allow encrypt-to to an disabled key; thus
              we pass 1 for the second last argument and 1 as the last
              argument to disable AKL. */
-          if ((rc = get_pubkey_byname (ctrl, GET_PUBKEY_NO_AKL,
-                                       NULL, pk, rov->d, NULL, NULL, 1)))
+          if ( (rc = get_pubkey_byname (ctrl,
+                                        NULL, pk, rov->d, NULL, NULL, 1, 1)) )
             {
               free_public_key ( pk ); pk = NULL;
               log_error (_("%s: skipped: %s\n"), rov->d, gpg_strerror (rc) );
@@ -1180,8 +1179,7 @@ build_pk_list (ctrl_t ctrl, strlist_t rcpts, PK_LIST *ret_pk_list)
           free_public_key (pk);
           pk = xmalloc_clear( sizeof *pk );
           pk->req_usage = PUBKEY_USAGE_ENC;
-          rc = get_pubkey_byname (ctrl, GET_PUBKEY_NORMAL,
-                                  NULL, pk, answer, NULL, NULL, 0);
+          rc = get_pubkey_byname (ctrl, NULL, pk, answer, NULL, NULL, 0, 0 );
           if (rc)
             tty_printf(_("No such user ID.\n"));
           else if ( !(rc=openpgp_pk_test_algo2 (pk->pubkey_algo,
@@ -1259,8 +1257,7 @@ build_pk_list (ctrl_t ctrl, strlist_t rcpts, PK_LIST *ret_pk_list)
 
       /* The default recipient is allowed to be disabled; thus pass 1
          as second last argument.  We also don't want an AKL. */
-      rc = get_pubkey_byname (ctrl, GET_PUBKEY_NO_AKL,
-                              NULL, pk, def_rec, NULL, NULL, 1);
+      rc = get_pubkey_byname (ctrl, NULL, pk, def_rec, NULL, NULL, 1, 1);
       if (rc)
         log_error(_("unknown default recipient \"%s\"\n"), def_rec );
       else if ( !(rc=openpgp_pk_test_algo2(pk->pubkey_algo,
@@ -1369,7 +1366,7 @@ build_pk_list (ctrl_t ctrl, strlist_t rcpts, PK_LIST *ret_pk_list)
    preference list, so I'm including it. -dms */
 
 int
-algo_available( preftype_t preftype, int algo, const struct pref_hint *hint)
+algo_available( preftype_t preftype, int algo, const union pref_hint *hint)
 {
   if( preftype == PREFTYPE_SYM )
     {
@@ -1395,26 +1392,16 @@ algo_available( preftype_t preftype, int algo, const struct pref_hint *hint)
     {
       if (hint && hint->digest_length)
 	{
-          unsigned int n = gcry_md_get_algo_dlen (algo);
-
-          if (hint->exact)
-            {
-              /* For example ECDSA requires an exact hash value so
-               * that we do not truncate.  For DSA we allow truncation
-               * and thus exact is not set.  */
-              if (hint->digest_length != n)
-                return 0;
-            }
-	  else if (hint->digest_length!=20 || opt.flags.dsa2)
+	  if (hint->digest_length!=20 || opt.flags.dsa2)
 	    {
 	      /* If --enable-dsa2 is set or the hash isn't 160 bits
 		 (which implies DSA2), then we'll accept a hash that
 		 is larger than we need.  Otherwise we won't accept
 		 any hash that isn't exactly the right size. */
-	      if (hint->digest_length > n)
+	      if (hint->digest_length > gcry_md_get_algo_dlen (algo))
 		return 0;
 	    }
-	  else if (hint->digest_length != n)
+	  else if (hint->digest_length != gcry_md_get_algo_dlen (algo))
 	    return 0;
 	}
 
@@ -1451,7 +1438,7 @@ algo_available( preftype_t preftype, int algo, const struct pref_hint *hint)
  */
 int
 select_algo_from_prefs(PK_LIST pk_list, int preftype,
-		       int request, const struct pref_hint *hint)
+		       int request, const union pref_hint *hint)
 {
   PK_LIST pkr;
   u32 bits[8];
@@ -1479,16 +1466,8 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
 	     --pgp2 mode is on.  This was a 2440 thing that was
 	     dropped from 4880 but is still relevant to GPG's 1991
 	     support.  All this doesn't mean IDEA is actually
-	     available, of course.
-
-             Because "de-vs" compliance will soon not anymore allow
-             3DES it does not make sense to assign 3DES as implicit
-             algorithm.  Instead it is better to use AES-128 as
-             implicit algorithm here.   */
-          if (opt.compliance == CO_DE_VS)
-            implicit = CIPHER_ALGO_AES;
-          else
-            implicit=CIPHER_ALGO_3DES;
+	     available, of course. */
+          implicit=CIPHER_ALGO_3DES;
 
 	  break;
 
@@ -1498,17 +1477,9 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
 	     code will never even be called.  Even if the hash wasn't
 	     locked at MD5, we don't support sign+encrypt in --pgp2
 	     mode, and that's the only time PREFTYPE_HASH is used
-	     anyway. -dms
+	     anyway. -dms */
 
-             Because "de-vs" compliance does not allow SHA-1 it does
-             not make sense to assign SHA-1 as implicit algorithm.
-             Instead it is better to use SHA-256 as implicit algorithm
-             (which will be the case for rfc4880bis anyway).  */
-
-          if (opt.compliance == CO_DE_VS)
-            implicit = DIGEST_ALGO_SHA256;
-          else
-            implicit = DIGEST_ALGO_SHA1;
+          implicit=DIGEST_ALGO_SHA1;
 
 	  break;
 
