@@ -30,20 +30,15 @@
 
 
 /* We keep all global options in the structure OPT.  */
-EXTERN_UNLESS_MAIN_MODULE
 struct
 {
   int verbose;
   unsigned int debug;
   int quiet;
   int use_sendmail;
-  int with_colons;
-  int no_autostart;
-  int add_revocs;
-  int realclean;
-  char *output;
-  char *gpg_program;
-  char *directory;
+  const char *output;
+  const char *gpg_program;
+  const char *directory;
   const char *default_from;
   strlist_t extra_headers;
 } opt;
@@ -81,9 +76,6 @@ struct uidinfo_list_s
   struct uidinfo_list_s *next;
   time_t created; /* Time the userid was created.  */
   char *mbox;  /* NULL or the malloced mailbox from UID.  */
-  unsigned int flags;  /* These flags are cleared on creation.  */
-  unsigned int expired:1;
-  unsigned int revoked:1;
   char uid[1];
 };
 typedef struct uidinfo_list_s *uidinfo_list_t;
@@ -95,29 +87,15 @@ void wks_set_status_fd (int fd);
 void wks_write_status (int no, const char *format, ...) GPGRT_ATTR_PRINTF(2,3);
 void free_uidinfo_list (uidinfo_list_t list);
 gpg_error_t wks_get_key (estream_t *r_key, const char *fingerprint,
-                         const char *addrspec, int exact, int binary);
+                         const char *addrspec, int exact);
 gpg_error_t wks_list_key (estream_t key, char **r_fpr,
                           uidinfo_list_t *r_mboxes);
 gpg_error_t wks_filter_uid (estream_t *r_newkey, estream_t key,
                             const char *uid, int binary);
-gpg_error_t wks_armor_key (estream_t *r_newkey, estream_t key,
-                           const char *prefix);
-gpg_error_t wks_find_add_revocs (estream_t key, const char *addrspec);
 gpg_error_t wks_send_mime (mime_maker_t mime);
 gpg_error_t wks_parse_policy (policy_flags_t flags, estream_t stream,
                               int ignore_unknown);
 void wks_free_policy (policy_flags_t policy);
-gpg_error_t wks_write_to_file (estream_t src, const char *fname);
-
-gpg_error_t wks_fname_from_userid (const char *userid, int hash_only,
-                                   char **r_fname, char **r_addrspec);
-gpg_error_t wks_compute_hu_fname (char **r_fname, const char *addrspec);
-gpg_error_t wks_install_key_core (estream_t key, const char *addrspec);
-gpg_error_t wks_cmd_install_key (const char *fname, const char *userid);
-gpg_error_t wks_cmd_remove_key (const char *userid);
-gpg_error_t wks_cmd_print_wkd_hash (const char *userid);
-gpg_error_t wks_cmd_print_wkd_url (const char *userid);
-
 
 /*-- wks-receive.c --*/
 
